#ifndef THREADPOOL_H
#define THREADPOOL_H

#include <vector>
#include <list>
#include <thread>
#include <condition_variable>
#include <queue>
#include <functional>
#include <atomic>

#include "spdlog/spdlog.h"
#include "fmt/std.h"

/**
 * @brief   1、在ThreadPool类中重新封装了一个OneThread类，类中有线程实例和相关的接口，每个OneThread实例就是一个线程
 *          2、采用可变参数模板函数添加任务到任务队列中
 *          3、线程池采用单例模式
 * 
 */

class ThreadPool
{
private:
    ThreadPool();
    ThreadPool(const ThreadPool &tp) = delete;
    ThreadPool &operator=(const ThreadPool &tp) = delete;


public:
    
    static ThreadPool& getInstance()
    {
        static ThreadPool tp;
        return tp;
    }

    ~ThreadPool();

    /**
     * @brief 向任务队列添加任务函数
     * 
     * @tparam F 函数（指针？）
     * @tparam Args 参数包
     * @param f 万能引用
     * @param args 万能引用
     */
    template <typename F, typename... Args>
    void add_Task(F &&f, Args &&...args)
    {
        /* 将函数参数和函数绑定 */
        std::function<void()> task = std::bind(std::forward<F>(f), std::forward<Args>(args)...);
        /* 作用域是用于解锁lock */
        {
            std::unique_lock<std::mutex> lock(m_mutexTask);
            m_queue_Tasks.emplace(std::move(task));         /* 入队 */
        }
        /* 唤醒一个线程 */
        m_cond_Task.notify_one();
    }

    /******  获取当前线程池在运行的线程个数，空闲线程的个数  ******/
    int getThreadMaxNum();                      /* 获取线程池最大线程的个数 */
    void setThreadMaxNum(int num);              /* 设置线程池最大线程的个数 */
    int getThreadMiniNum();                     /* 获取线程池最小线程的个数 */
    void setThreadMiniNum(int num);             /* 设置线程池最小线程的个数 */
    int getThreadIdleNum();                     /* 获取线程池空闲线程的个数 */
    int getThreadRunNum();                      /* 获取线程池正在运行的线程个数 */
    int getThreadLiveNum();                     /* 获取线程池现存的线程个数 */
    int getThreadAddNum();                      /* 线程池每次创建线程的个数 */
    void setThreadAddNum(int num);              /* 设置线程池每次创建线程的个数 */
    int getThreadMiniIdle();                    /* 线程池最小空闲线程的个数 */
    void setThreadMiniIdle(int num);            /* 设置线程池最小空闲线程的个数 */
    int getThreadMaxIdle();                     /* 线程池最大空闲线程的个数 */
    void setThreadMaxIdle(int num);             /* 设置线程池最大空闲线程的个数 */

private:
    void managerThread();                       /* 维护线程池的线程，如提前创建线程，销毁线程 */
    void createThread(int num);                 /* 创建新的线程 */
    // void do_work();                          /* 工作线程函数，这里没有使用，如果需要使用，需要将其变成全局函数 */
    void clearThread();                         /* 清除失效的线程实例 */

private:
    class OneThread;                                    /* 类声明 */
    // std::list<std::thread> m_threads;                   /* 线程数组，维护线程池 */
    std::list<OneThread> m_threads;                     /* 线程数组，类型为OneThread，一个实例包含一个线程 */
    std::queue<std::function<void(void)>> m_queue_Tasks;/* 任务队列，任务函数都是无返回值的函数 */
    std::condition_variable m_cond_Task;                /* 条件变量，没有任务的时候阻塞住 */

    bool m_stop;                                        /* 线程是否停止 */
    std::mutex m_mutexTask;                             /* 互斥锁，搭配环境变量使用，对任务队列上锁 */
    std::thread m_managerThread;                        /* 管理线程 */

    int m_threadMaxNum = 256;                               /* 默认最大线程数 */
    int m_threadMiniNum = 5;                                /* 默认线程池中最小线程数 */
    std::atomic<int> m_threadRunNum;                        /* 正在运行的线程个数 */
    std::atomic<int> m_threadLiveNum;                       /* 现有的线程个数 */
    std::atomic<int> m_threadExitNum;                       /* 需要销毁的线程个数 */
    std::atomic<int> m_threadAddNum;                        /* 每次需要添加的线程个数 */
    std::atomic<int> m_threadMiniIdle;                      /* 如果空闲线程小于这个数，就添加m_ThreadAddNum个线程 */
    std::atomic<int> m_threadMaxIdle;                       /* 最大空闲的线程个数，超过这个个数一定时间，就会销毁多余的 */

    /**
     * @brief 封装工作函数，加入线程的运行状态，方便线程池查看线程的状态
     * 
     */
    class OneThread
    {
        private:
            std::thread::id m_threadID;     /* 线程ID */
            ThreadPool *m_tp = nullptr;     /* 线程池指针 */
            std::atomic<bool> m_isLive;     /* 线程是否存活 */
            std::thread t;                  /* 线程 */

            void task()
            {
                m_tp->m_threadLiveNum++;
                while (1)
                {
                    /* 等待任务队列中有任务 */
                    std::unique_lock<std::mutex> lock(m_tp->m_mutexTask);
                    m_tp->m_cond_Task.wait(lock, [this]
                                           { return !m_tp->m_queue_Tasks.empty() || m_tp->m_stop ||
                                                    (m_tp->m_threadExitNum > 0); });
                    /* 任务队列中有任务了，条件变量被唤醒了，先判断是不是需要结束线程 */
                    if (m_tp->m_stop && m_tp->m_queue_Tasks.empty())
                    {
                        break;
                    }
                    // log_d("判断是否需要退出线程。。。");
                    /* 判断是不是需要销毁多余的线程 */
                    if (m_tp->m_threadExitNum.load() > 0 )
                    {
                        // log_d("线程需要退出，再次判断任务队列中有没有任务。。。");
                        m_tp->m_threadExitNum--;
                        /* 再次判断有没有新任务，有就不退出 */
                        if ( m_tp->m_queue_Tasks.empty())
                        {
                            // log_d("线程:%d即将退出。。。", m_threadID);
                            break;
                        }
                    }
                    /* 取出任务，执行任务 */
                    std::function<void()> task(std::move(m_tp->m_queue_Tasks.front()));
                    m_tp->m_queue_Tasks.pop();  /* 取出的任务出队 */
                    lock.unlock();              /* 解锁任务队列 */

                    m_tp->m_threadRunNum++;     /* 更新线程状态数 */
                    /* 开始执行任务 */
                    task();

                    m_tp->m_threadRunNum--;         /* 更新线程状态数 */
                }
                /* 线程结束 */
                m_tp->m_threadLiveNum--;
                this->m_isLive = false;
                std::stringstream ss;
                ss << m_threadID;
                SPDLOG_DEBUG("线程ID:{}结束", ss.str());
                return;
            }

        public: 
            /* 构造函数 */
            OneThread(ThreadPool* tp)
            {
                m_tp = tp;
                this->m_isLive = true;
                run();
            }
            /* 析构函数 */
            ~OneThread() { t.join(); }
            /* 线程工作函数 */
            inline void run()
            {
                t = std::thread(&OneThread::task, this);
                m_threadID = t.get_id();
            }
            bool isLive() { return m_isLive; }
            // void setIsLive(bool status) { m_isLive = status; }
            std::thread::id getID() { return m_threadID; }
    };

};

#endif /* THREADPOOL_H */